pro tsys_ant

common global
common data_set

; tsys_ant.pro
; 
; This procedure converts SMA baseline-based Tsys into antenna-based Tsys.
;      Tsys for ant 1 stored in : bl[pbf].csnr
;      Tsys for ant 2 stored in : bl[pbf].cnoise
; *** IMPORTANT *** Do not modify Tsys values AT ALL before using this 
;      procedure!
; 
; For use with the procedures tsys_replace (to replace one antenna's 
; Tsys by another's) and tsys_bsl (to convert the antenna-based Tsys
; back to a baseline-based quantity for use with other routines).
;
;  mhughes 28Jan07

result=dat_filter(s_f,/reset,/no_notify)

; check which antennas are available
subset1=bl[pbf].itel1
subset2=bl[pbf].itel2
subset=[subset1,subset2]
ants=subset(uniq(subset, sort(subset)))
nants=n_elements(ants)
if (n_elements(ants) lt 3) then begin
  print,"***************************************************************"
  print,"Not enough antennas in the array.  Revise selection criteria."
  print,"Cannot extract antenna-based values with fewer than 3 antennas."
  print,"***************************************************************"
  return
endif
print,"****************************************************************"
print,"Recovering antenna-based Tsys..."

; Making a new array of integration number and Tsys
loc=where(bl[pbf].itel1 eq ants(0) and bl[pbf].itel2 eq ants(1))
dimens=n_elements(loc)
; Tsys array will store Tsys values for each antenna for each integration, 
; spectral channel, sideband, polarization, etc.
tsysarray=fltarr(dimens,nants)
; temparray will store baseline-based Tsys values
temparray=fltarr(dimens,nants,nants)
for i=0,nants-2 do begin
  for j=i+1,nants-1 do begin
    loc=where(bl[pbf].itel1 eq ants(i) and bl[pbf].itel2 eq ants(j),count)
    if count ne dimens then begin
       print,'Data points not consistent within baselines.'
       print,'FAILED !'
       return
    endif
    temparray(*,i,j)=sp[psf[loc]].tssb
  endfor
endfor

; calculating antenna-based Tsys
t12=temparray(*,0,1)
t13=temparray(*,0,2)
t23=temparray(*,1,2)
tsysarray(*,0)=t12*t13/t23
tsysarray(*,1)=t12*t23/t13
tsysarray(*,2)=t13*t23/t12
if nants gt 3 then begin
  for i=3,nants-1 do begin
    t1n=temparray(*,0,i)
    t2n=temparray(*,1,i)
    tsysarray(*,i)=t1n*t2n/t12
  endfor
endif

; putting antenna-based Tsys into variables csnr and cnoise
for i=0,nants-2 do begin
  for j=i+1,nants-1 do begin
        loc=where(bl[pbf].itel1 eq ants(i) and bl[pbf].itel2 eq ants(j))
        bl[pbf[loc]].csnr=tsysarray(*,i)
        bl[pbf[loc]].cnoise=tsysarray(*,j)
  endfor
endfor

print,"***************************************************************"
print,'FINISHED'
print,'antenna (itel1) Tsys stored in bl[pbf].csnr'
print,'antenna (itel2) Tsys stored in bl[pbf].cnoise'
print,"***************************************************************"
  
return
end
